@@ -306,8 +306,10 @@ hd_home_desktop_do_motion (HdHome *home,
       GList *next = list->next;
       /* '&& next' ensures that we always average at least one previous
        * movement event to get our velocity, even if it is too old. */
-      if (time > HDH_PAN_VELOCITY_HISTORY && next)
-        priv->drag_list = g_list_remove_link(priv->drag_list, list);
+      if (time > HDH_PAN_VELOCITY_HISTORY && next) {
+        g_free (list->data);
+        priv->drag_list = g_list_delete_link(priv->drag_list, list);
+      }
       else
         {
           HdHomeDrag *drag = (HdHomeDrag*)list->data;
@@ -717,7 +719,7 @@ root_window_client_message (XClientMessageEvent *event, HdHome *home)
 
 	      gdk_pixbuf_unref (pixbuf);
 	    }
-	    return;
+	    break;
 	  case 1:
 	    unlink (filename);
 	    break;
@@ -854,8 +856,8 @@ hd_home_constructed (GObject *object)
   priv->desktop = XCreateWindow (wm->xdpy,
 				 wm->root_win->xwindow,
 				 0, 0,
-				 wm->xdpy_width,
-				 wm->xdpy_height,
+				 HD_COMP_MGR_LANDSCAPE_WIDTH,
+				 HD_COMP_MGR_LANDSCAPE_WIDTH,
 				 0,
 				 CopyFromParent,
 				 InputOnly,
