@@ -57,7 +57,6 @@
 #else
 # define VISIBILITY(...)  /* NOP */
 #endif
-
 /* ------------------------------------------------------------------------- */
 #define I_(str) (g_intern_static_string ((str)))
 
@@ -94,7 +93,7 @@ hd_render_manager_state_get_type (void)
 
 /* ------------------------------------------------------------------------- */
 
-G_DEFINE_TYPE (HdRenderManager, hd_render_manager, CLUTTER_TYPE_GROUP);
+G_DEFINE_TYPE (HdRenderManager, hd_render_manager, TIDY_TYPE_CACHED_GROUP);
 #define HD_RENDER_MANAGER_GET_PRIVATE(obj) \
                 (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
                 HD_TYPE_RENDER_MANAGER, HdRenderManagerPrivate))
@@ -738,7 +737,8 @@ hd_render_manager_unzoom_background()
 }
 
 /* Checks the whole tree for visibility */
-static gboolean actor_is_visible(ClutterActor *actor) {
+gboolean hd_render_manager_actor_is_visible(ClutterActor *actor)
+{
   ClutterActor *stage = clutter_actor_get_stage(actor);
   for (;actor;actor=clutter_actor_get_parent(actor))
     if (actor!=stage && !CLUTTER_ACTOR_IS_VISIBLE(actor))
@@ -747,7 +747,7 @@ static gboolean actor_is_visible(ClutterActor *actor) {
   return TRUE;
 }
 
-static void
+void
 hd_render_manager_set_input_viewport()
 {
   ClutterGeometry geom[HDRM_BUTTON_COUNT + 1];
@@ -768,7 +768,7 @@ hd_render_manager_set_input_viewport()
           ClutterActor *button;
           button = hd_render_manager_get_button((HDRMButtonEnum)i);
           if (button &&
-              actor_is_visible(button) &&
+              hd_render_manager_actor_is_visible(button) &&
               (CLUTTER_ACTOR_IS_REACTIVE(button)) &&
               (i!=HDRM_BUTTON_BACK || !app_mode))
             {
@@ -785,7 +785,7 @@ hd_render_manager_set_input_viewport()
          /* also in the case of "dialog blur": */
          || (priv->state == HDRM_STATE_APP
               && priv->status_area
-              && actor_is_visible(priv->status_area)
+              && hd_render_manager_actor_is_visible(priv->status_area)
               /* FIXME: the following check does not work when there are
                * two levels of dialogs */
               && (priv->current_blur == HDRM_BLUR_BACKGROUND ||
@@ -1004,7 +1004,7 @@ void hd_render_manager_sync_clutter_before ()
     hd_render_manager_set_visibilities();
 
   /* Now look at what buttons we have showing, and add each visible button X
-   * to the X input viewport */
+   * to the X input viewport. FIXME: Do we need this now HdTitleBar does it? */
   hd_render_manager_set_input_viewport();
 
   /* as soon as we start a transition, set out left-hand button to be
@@ -1306,6 +1306,7 @@ MBWindowManagerClient *hd_render_manager_get_status_area_client(void)
 void hd_render_manager_set_visible(HDRMButtonEnum button, gboolean visible)
 {
   HdRenderManagerPrivate *priv = the_render_manager->priv;
+  /* FIXME: Should this be handled by HdTitleBar? */
 
   switch (button)
   {
@@ -1681,6 +1682,13 @@ inline HDRMStateEnum hd_render_manager_get_state()
   return the_render_manager->priv->state;
 }
 
+inline HDRMStateEnum hd_render_manager_get_previous_state()
+{
+  if (!the_render_manager)
+    return HDRM_STATE_UNDEFINED;
+  return the_render_manager->priv->previous_state;
+}
+
 static const char *hd_render_manager_state_str(HDRMStateEnum state)
 {
   GTypeClass *state_class = g_type_class_ref (HD_TYPE_RENDER_MANAGER_STATE);
@@ -1824,6 +1832,46 @@ hd_render_manager_clip_geo(ClutterGeometry *geo)
   return TRUE;
 }
 
+/* Tries to guess whether @geo was meant for a @scrw x @scrh screen. */
+static gboolean
+hd_render_manager_geo_is_suitable_for_screen(const ClutterGeometry *geo,
+                                             guint scrw, guint scrh)
+{
+  if (geo->x + (gint)geo->width  > (gint)scrw)
+    return FALSE;
+  if (geo->y + (gint)geo->height > (gint)scrh)
+    return FALSE;
+  return TRUE;
+}
+
+/*
+ * Returns @actor's %ClutterGeometry.  If it finds that it is rotated
+ * it maps @geo to the screens current orientation.  This allows to
+ * compare geometries of all clients whether they're layed out for
+ * portrait or landscape.  Of course all heuristics can break.
+ */
+static void
+hd_render_manager_get_geo_for_current_screen(ClutterActor *actor,
+                                             ClutterGeometry *geo)
+{
+  guint scrw, scrh;
+  ClutterGeometry rgeo;
+
+  scrw = hd_comp_mgr_get_current_screen_width();
+  scrh = hd_comp_mgr_get_current_screen_height();
+  clutter_actor_get_geometry(actor, geo);
+
+  /* Is @geo ok for current orientation? */
+  if (hd_render_manager_geo_is_suitable_for_screen(geo, scrw, scrh))
+    return;
+
+  /* If the rotated geometry is ok, return that, otherwise give it up. */
+  rgeo = *geo;
+  hd_util_rotate_geometry(&rgeo, scrw, scrh);
+  if (hd_render_manager_geo_is_suitable_for_screen(&rgeo, scrw, scrh))
+    *geo = rgeo;
+}
+
 /* Called to restack the windows in the way we use for rendering... */
 void hd_render_manager_restack()
 {
@@ -1927,7 +1975,7 @@ void hd_render_manager_restack()
             ClutterGeometry geo = {0};
             gboolean maximized;
 
-            clutter_actor_get_geometry(child, &geo);
+            hd_render_manager_get_geo_for_current_screen(child, &geo);
             if (!hd_render_manager_clip_geo(&geo))
               /* It's neiteher maximized nor @app_top, it doesn't exist. */
               continue;
@@ -2348,13 +2396,13 @@ void hd_render_manager_append_geo_cb(ClutterActor *actor, gpointer data)
   GList **list = (GList**)data;
   if (hd_render_manager_actor_opaque(actor))
     {
-      ClutterGeometry *geo = g_malloc(sizeof(ClutterGeometry));
-      clutter_actor_get_geometry(actor, geo);
+      ClutterGeometry geo;
 
-      if (!hd_render_manager_clip_geo (geo))
+      hd_render_manager_get_geo_for_current_screen(actor, &geo);
+      if (!hd_render_manager_clip_geo (&geo))
         return;
-      *list = g_list_prepend(*list, geo);
-      VISIBILITY ("BLOCKER %dx%d%+d%+d", MBWM_GEOMETRY(geo));
+      *list = g_list_prepend(*list, g_memdup(&geo, sizeof(geo)));
+      VISIBILITY ("BLOCKER %dx%d%+d%+d", MBWM_GEOMETRY(&geo));
     }
 }
 
@@ -2412,7 +2460,7 @@ void hd_render_manager_set_visibilities()
 	  if (hd_render_manager_should_ignore_actor(child))
 	      continue;
 
-          clutter_actor_get_geometry(child, &geo);
+          hd_render_manager_get_geo_for_current_screen(child, &geo);
           /*TEST clutter_actor_set_opacity(child, 63);*/
           VISIBILITY ("IS %p (%dx%d%+d%+d) VISIBLE?", child, MBWM_GEOMETRY(&geo));
           if (hd_render_manager_is_visible(blockers, geo))
@@ -2557,8 +2605,9 @@ void hd_render_manager_place_titlebar_elements (void)
 
   x = 0;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE(priv->button_task_nav)
-      || CLUTTER_ACTOR_IS_VISIBLE(priv->button_launcher))
+  /* Check whether buttons are visible based on state rather than anything
+   * else. As actual actor visibilities may only be set on idle. */
+  if (hd_title_bar_get_state(priv->title_bar) & HDTB_VIS_BTN_LEFT_MASK)
     x += hd_title_bar_get_button_width(priv->title_bar);
 
   if (priv->status_area && CLUTTER_ACTOR_IS_VISIBLE(priv->status_area))
@@ -2700,5 +2749,6 @@ void hd_render_manager_remove_input_blocker() {
     * that the dbus signal arrived after the window was mapped. This has
     * had to be extended because under system load from things like flash,
     * dbus is so slow the message will often take >250ms. bug 128009 */
-   return hd_comp_mgr_time_since_last_map(priv->comp_mgr) > 1000;
+   return !hd_comp_mgr_is_portrait()
+     && hd_comp_mgr_time_since_last_map(priv->comp_mgr) > 1000;
  }
