@@ -109,6 +109,8 @@ static struct
   /*
    * What is *_fsm() currently doing:
    * -- #IDLE:      nothing, we're sitting in landscape or portrait
+   * -- #TRANS_START: we're hoping to be called back on idle to start the
+   *                  animation
    * -- #FADE_OUT:  hd_transition_fade_and_rotate() is fading out
    * -- #WAITING:   for X to finish reconfiguring the screen
    * -- #FADE_IN:   second hd_transition_fade_and_rotate() is in progress
@@ -116,6 +118,7 @@ static struct
   enum
   {
     IDLE,
+    TRANS_START,
     FADE_OUT,
     WAIT_FOR_ROOT_CONFIG,
     WAIT_FOR_DAMAGES,
@@ -139,11 +142,6 @@ static struct
   /* This timer counts from when we first entered the WAITING state,
    * so if we are continually getting damage we don't just hang there. */
   GTimer *timer;
-
-  /* If a client (like CallUI) was forcing portrait mode and it quits,
-   * we leave it visible and put an HDEffectData this list. During blanking we
-   * remove the actor by calling hd_transition_completed. */
-  GList *effects_waiting;
 } Orientation_change;
 
 /* If %TRUE keep reloading transitions.ini until we can
@@ -317,7 +315,7 @@ on_decor_progress_timeline_new_frame(ClutterTimeline *timeline,
                                      ClutterActor *progress_texture)
 {
   if (!hd_dbus_display_is_off && TIDY_IS_SUB_TEXTURE(progress_texture) &&
-      CLUTTER_ACTOR_IS_VISIBLE(progress_texture))
+      hd_render_manager_actor_is_visible(progress_texture))
     {
       /* The progress animation is a series of frames packed
        * into a texture - like a film strip
@@ -330,10 +328,10 @@ on_decor_progress_timeline_new_frame(ClutterTimeline *timeline,
           TIDY_SUB_TEXTURE(progress_texture),
           &progress_region);
 
-      /* FIXME: We really want to set this to queue damage with an area -
-       * like we do for windows. Otherwise we end up updating the whole
-       * screen for this. */
-      clutter_actor_queue_redraw(progress_texture);
+      /* We just queue damage with an area like we do for windows.
+       * Otherwise we end up updating the whole screen for this.
+       * This also takes account of visibility. */
+      hd_util_partial_redraw_if_possible(progress_texture, 0);
     }
 }
 
@@ -348,6 +346,7 @@ on_popup_timeline_new_frame(ClutterTimeline *timeline,
   gboolean pop_top, pop_bottom; /* pop in from the top, or the bottom */
 
   float overshoot;
+  ClutterGeometry geo;
 
   actor = data->cclient_actor;
   if (!CLUTTER_IS_ACTOR(actor))
@@ -355,12 +354,7 @@ on_popup_timeline_new_frame(ClutterTimeline *timeline,
   filler = data->particles[0];
 
   /* We need to get geometry each frame as often windows have
-   * a habit of changing size while they move. If we have filler
-   * we remove it first, so it doesn't affect the geometry. */
-  if (filler && clutter_actor_get_parent(filler))
-    clutter_container_remove_actor(
-        CLUTTER_CONTAINER(clutter_actor_get_parent(filler)), filler);
-  ClutterGeometry geo;
+   * a habit of changing size while they move. */
   clutter_actor_get_geometry(actor, &geo);
 
   pop_top = geo.y==0;
@@ -397,13 +391,9 @@ on_popup_timeline_new_frame(ClutterTimeline *timeline,
 
   /* use a slither of filler to fill in the gap where the menu
    * has jumped a bit too far up */
-  if (filler &&
-      ((status_pos>status_high && pop_top) ||
-       (status_pos<status_high && pop_bottom)))
+  if ((status_pos>status_high && pop_top) ||
+       (status_pos<status_high && pop_bottom))
     {
-      // re-add the filler (see above)
-      if (CLUTTER_IS_CONTAINER(actor))
-        clutter_container_add_actor(CLUTTER_CONTAINER(actor), filler);
       clutter_actor_show(filler);
       if (pop_top)
         {
@@ -424,6 +414,8 @@ on_popup_timeline_new_frame(ClutterTimeline *timeline,
                     CLUTTER_FLOAT_TO_FIXED(status_high-status_pos));
         }
     }
+  else
+    clutter_actor_hide(filler);
 }
 
 static void
@@ -438,6 +430,16 @@ on_fade_timeline_new_frame(ClutterTimeline *timeline,
   if (!CLUTTER_IS_ACTOR(actor))
     return;
 
+  if (hd_dbus_display_is_off)
+    {
+      /* skip the animation */
+      if (data->event == MBWMCompMgrClientEventUnmap)
+        clutter_actor_set_opacity(actor, 0);
+      else
+        clutter_actor_set_opacity(actor, 255);
+      return;
+    }
+
   amt =  (float)clutter_timeline_get_progress(timeline);
   /* reverse if we're removing this */
   if (data->event == MBWMCompMgrClientEventUnmap)
@@ -469,6 +471,15 @@ on_close_timeline_new_frame(ClutterTimeline *timeline,
   if (!CLUTTER_IS_ACTOR(actor))
     return;
 
+  if (hd_dbus_display_is_off && data)
+    {
+      for (i = 0; i < HDCM_UNMAP_PARTICLES; ++i)
+        if (data->particles[i])
+	  clutter_actor_hide (data->particles[i]);
+      clutter_actor_hide (actor);
+      return;
+    }
+
   amt = (float)clutter_timeline_get_progress(timeline);
 
   amtx = 1.6 - amt*2.5; // shrink in x
@@ -539,7 +550,7 @@ on_notification_timeline_new_frame(ClutterTimeline *timeline,
   guint width, height;
 
   actor = data->cclient_actor;
-  if (!CLUTTER_IS_ACTOR(actor))
+  if (!CLUTTER_IS_ACTOR(actor) || hd_dbus_display_is_off)
     return;
 
   clutter_actor_get_size(actor, &width, &height);
@@ -717,12 +728,13 @@ on_rotate_screen_timeline_new_frame(ClutterTimeline *timeline,
   n_frames = clutter_timeline_get_n_frames(timeline);
   amt = frame_num / (float)n_frames;
   // we want to ease in, but speed up as we go - X^3 does this nicely
-  amt = amt*amt*amt;
+  amt = amt*amt;
   if (data->event == MBWMCompMgrClientEventUnmap)
     amt = 1-amt;
   /* dim=1 -> screen is black, dim=0 -> normal. Only
-   * dim out right at the end of the animation */
-  dim_amt = amt*4 - 3;
+   * dim out right at the end of the animation - and
+   * then only by half */
+  dim_amt = amt*2 - 1.5;
   if (dim_amt<0)
     dim_amt = 0;
   angle = data->angle * amt;
@@ -749,7 +761,6 @@ static void
 on_screen_size_changed (ClutterActor *stage, GParamSpec *unused,
                         HDEffectData *data)
 {
-  gint tmp;
   guint scrw, scrh;
   ClutterActor *actor;
 
@@ -758,27 +769,10 @@ on_screen_size_changed (ClutterActor *stage, GParamSpec *unused,
   clutter_actor_get_size (stage, &scrw, &scrh);
   actor = mb_wm_comp_mgr_clutter_client_get_actor (data->cclient);
 
-  /* It is very interesting to observe the dualism here. */
-  if (scrw > scrh)
-    { /* Coming from portrait to landscape. */
-      clutter_actor_set_rotation (actor, CLUTTER_Z_AXIS, -90, 0, 0, 0);
-
-      tmp = data->geo.x;
-      data->geo.x = data->geo.y;
-      data->geo.y = scrh - (tmp + data->geo.width);
-    }
+  if (hd_util_rotate_geometry(&data->geo, scrw, scrh))
+    clutter_actor_set_rotation (actor, CLUTTER_Z_AXIS, -90, 0, 0, 0);
   else
-    { /* Coming from landscape to portrait. */
-      clutter_actor_set_rotation (actor, CLUTTER_Z_AXIS, +90, 0, 0, 0);
-
-      tmp = data->geo.y;
-      data->geo.y = data->geo.x;
-      data->geo.x = scrw - (tmp + data->geo.height);
-    }
-
-  tmp = data->geo.width;
-  data->geo.width = data->geo.height;
-  data->geo.height = tmp;
+    clutter_actor_set_rotation (actor, CLUTTER_Z_AXIS, +90, 0, 0, 0);
 
   clutter_actor_set_position (actor,
                               data->geo.x + data->geo.width/2,
@@ -875,14 +869,25 @@ hd_transition_popup(HdCompMgr                  *mgr,
   data->cclient = mb_wm_object_ref (MB_WM_OBJECT (cclient));
   data->cclient_actor = g_object_ref (actor);
   data->hmgr = HD_COMP_MGR (mgr);
-  data->timeline =
-        g_object_ref( hd_transition_timeline_new("popup", event, 250) );
+  data->timeline = hd_transition_timeline_new("popup", event, 250);
   g_signal_connect (data->timeline, "new-frame",
                         G_CALLBACK (on_popup_timeline_new_frame), data);
   g_signal_connect (data->timeline, "completed",
                         G_CALLBACK (hd_transition_completed), data);
   data->geo = geo;
 
+  /*
+   * If @actor is a fullscreen dialog it's not in apptop but in home_blur.
+   * This case when its client is unmapped the next client is stacked on
+   * the top of it, hiding this @actor, which we want to keep shown until
+   * the end of transition.
+   */
+  if (event == MBWMCompMgrClientEventUnmap)
+    {
+      hd_render_manager_return_dialog(actor);
+      clutter_actor_show(actor);
+    }
+
   mb_wm_comp_mgr_clutter_client_set_flags (cclient,
                               MBWMCompMgrClutterClientDontUpdate |
                               MBWMCompMgrClutterClientEffectRunning);
@@ -891,6 +896,7 @@ hd_transition_popup(HdCompMgr                  *mgr,
 
   /* Add actor for the background when we pop a bit too far */
   data->particles[0] = g_object_ref(clutter_rectangle_new());
+  clutter_container_add_actor(CLUTTER_CONTAINER(actor), data->particles[0]);
   hd_gtk_style_get_bg_color(HD_GTK_BUTTON_SINGLETON, GTK_STATE_NORMAL,
                               &col);
   clutter_rectangle_set_color(CLUTTER_RECTANGLE(data->particles[0]),
@@ -918,8 +924,7 @@ hd_transition_fade(HdCompMgr                  *mgr,
   data->cclient_actor = g_object_ref (
       mb_wm_comp_mgr_clutter_client_get_actor( data->cclient ) );
   data->hmgr = HD_COMP_MGR (mgr);
-  data->timeline =
-        g_object_ref( hd_transition_timeline_new("fade", event, 250) );
+  data->timeline = hd_transition_timeline_new("fade", event, 250);
 
   if (HD_IS_BANNER_NOTE(c))
     {
@@ -967,8 +972,7 @@ hd_transition_fade_out_loading_screen(ClutterActor *loading_image)
     data->event = MBWMCompMgrClientEventUnmap;
     data->cclient_actor = g_object_ref ( loading_image );
     data->hmgr = 0;
-    data->timeline = g_object_ref(
-        clutter_timeline_new_for_duration ( duration ) );
+    data->timeline = clutter_timeline_new_for_duration ( duration );
     data->final_alpha = 1;
     /* the delay before we start to fade out. We implement this by setting
      * the final_alpha value to something *past* opaque */
@@ -1050,9 +1054,8 @@ hd_transition_close_app (HdCompMgr                  *mgr,
   data->cclient = mb_wm_object_ref (MB_WM_OBJECT (cclient));
   data->cclient_actor = g_object_ref (actor);
   data->hmgr = HD_COMP_MGR (mgr);
-  data->timeline = g_object_ref(
-                clutter_timeline_new_for_duration (
-                    hd_transition_get_int("app_close", "duration", 500) ) );
+  data->timeline = clutter_timeline_new_for_duration (
+                    hd_transition_get_int("app_close", "duration", 500));
   g_signal_connect (data->timeline, "new-frame",
                     G_CALLBACK (on_close_timeline_new_frame), data);
   g_signal_connect (clutter_stage_get_default (), "notify::allocation",
@@ -1091,60 +1094,6 @@ hd_transition_close_app (HdCompMgr                  *mgr,
   hd_transition_play_sound (HDCM_WINDOW_CLOSED_SOUND);
 }
 
-void
-hd_transition_close_app_before_rotate (HdCompMgr                  *hmgr,
-                                       MBWindowManagerClient      *c)
-{
-  MBWMClientType c_type = MB_WM_CLIENT_CLIENT_TYPE (c);
-  MBWMCompMgrClutterClient * cclient;
-  HDEffectData             * data;
-  ClutterActor             * actor;
-  ClutterContainer         * parent;
-
-  /* proper app close animation */
-  if (c_type != MBWMClientTypeApp)
-    return;
-
-  /* The switcher will do the effect if it's active,
-   * don't interfere. */
-  if (hd_render_manager_get_state()==HDRM_STATE_TASK_NAV)
-    return;
-
-  cclient = MB_WM_COMP_MGR_CLUTTER_CLIENT (c->cm_client);
-  actor = mb_wm_comp_mgr_clutter_client_get_actor (cclient);
-  if (!actor || !CLUTTER_ACTOR_IS_VISIBLE(actor))
-    return;
-
-  data = g_new0 (HDEffectData, 1);
-  data->event = MBWMCompMgrClientEventUnmap;
-  data->cclient = mb_wm_object_ref (MB_WM_OBJECT (cclient));
-  data->cclient_actor = g_object_ref (actor);
-  data->hmgr = hmgr;
-
-
-  mb_wm_comp_mgr_clutter_client_set_flags (cclient,
-                                    MBWMCompMgrClutterClientDontUpdate |
-                                    MBWMCompMgrClutterClientDontShow |
-                                    MBWMCompMgrClutterClientEffectRunning);
-
-  /* reparent our actor so it will be visible when we switch views */
-  parent = hd_render_manager_get_front_group();
-  clutter_actor_reparent(actor,
-      CLUTTER_ACTOR(parent));
-  clutter_actor_lower_bottom(actor);
-  /* Also add a fake titlebar background, as the real one will disappear
-   * immediately because the app has closed. */
-  data->particles[0] = g_object_ref(hd_title_bar_create_fake(
-                                         HD_COMP_MGR_LANDSCAPE_HEIGHT));
-  clutter_container_add_actor(parent, data->particles[0]);
-
-  Orientation_change.effects_waiting = g_list_append(
-      Orientation_change.effects_waiting, data);
-  hd_comp_mgr_set_effect_running(hmgr, TRUE);
-
-  hd_transition_play_sound (HDCM_WINDOW_CLOSED_SOUND);
-}
-
 void
 hd_transition_notification(HdCompMgr                  *mgr,
                            MBWindowManagerClient      *c,
@@ -1165,8 +1114,7 @@ hd_transition_notification(HdCompMgr                  *mgr,
   data->cclient_actor = g_object_ref (
       mb_wm_comp_mgr_clutter_client_get_actor( data->cclient ) );
   data->hmgr = HD_COMP_MGR (mgr);
-  data->timeline =
-        g_object_ref( hd_transition_timeline_new("notification", event, 500) );
+  data->timeline = hd_transition_timeline_new("notification", event, 500);
 
   g_signal_connect (data->timeline, "new-frame",
                         G_CALLBACK (on_notification_timeline_new_frame), data);
@@ -1298,8 +1246,7 @@ hd_transition_subview(HdCompMgr                  *mgr,
   data->cclient2_actor = g_object_ref (
       mb_wm_comp_mgr_clutter_client_get_actor( data->cclient2 ) );
   data->hmgr = HD_COMP_MGR (mgr);
-  data->timeline =
-        g_object_ref( hd_transition_timeline_new("subview", event, 250) );
+  data->timeline = hd_transition_timeline_new("subview", event, 250);
 
   g_signal_connect (data->timeline, "new-frame",
                         G_CALLBACK (on_subview_timeline_new_frame), data);
@@ -1359,8 +1306,7 @@ hd_transition_fade_and_rotate(gboolean first_part,
   HDEffectData *data = g_new0 (HDEffectData, 1);
   data->event = first_part ? MBWMCompMgrClientEventMap :
                              MBWMCompMgrClientEventUnmap;
-  data->timeline =
-        g_object_ref( hd_transition_timeline_new("rotate", data->event, 300) );
+  data->timeline = hd_transition_timeline_new("rotate", data->event, 300);
 
   g_signal_connect (data->timeline, "new-frame",
                         G_CALLBACK (on_rotate_screen_timeline_new_frame), data);
@@ -1425,13 +1371,41 @@ hd_transition_rotating_fsm(void)
   switch (Orientation_change.phase)
     {
       case IDLE:
+        Orientation_change.phase = TRANS_START;
+        Orientation_change.direction = Orientation_change.new_direction;
+        /* Take a screenshot of the screen as we currently are... */
+        tidy_cached_group_changed(CLUTTER_ACTOR(hd_render_manager_get()));
+        tidy_cached_group_set_render_cache(
+          CLUTTER_ACTOR(hd_render_manager_get()), 1);
+        tidy_cached_group_set_downsampling_factor(
+          CLUTTER_ACTOR(hd_render_manager_get()), 1);
+        /* Super massive extra large hack to remove status area when launching
+         * phone app from the launcher */
+        if (hd_render_manager_get_previous_state()==HDRM_STATE_LAUNCHER)
+          clutter_actor_hide(hd_render_manager_get_status_area());
+        /* Force redraw for screenshot *now*, before windows have a
+         * chance to change */
+        clutter_redraw(CLUTTER_STAGE(clutter_stage_get_default()));
+        /* Start rotate transition */
+        hd_util_set_rotating_property(Orientation_change.wm, TRUE);
+        /* Layout windows at the rotated size */
+        hd_util_set_screen_size_properties(Orientation_change.wm,
+            Orientation_change.direction == GOTO_PORTRAIT ? 480 : 800,
+            Orientation_change.direction == GOTO_PORTRAIT ? 800 : 480);
+        Orientation_change.wm->flags |= MBWindowManagerFlagLayoutRotated;
+        mb_wm_layout_update(Orientation_change.wm->layout);
+        /* We now call ourselves back on idle. The idea is that the sudden
+         * influx of X events from resizing kills our animation as we don't
+         * get to idle for a while. So only start the transition once we
+         * got to idle at least once! */
+        g_idle_add((GSourceFunc)(hd_transition_rotating_fsm), NULL);
+        break;
+      case TRANS_START:
         /* Fade to black ((c) Metallica) */
         Orientation_change.phase = FADE_OUT;
-        Orientation_change.direction = Orientation_change.new_direction;
-        hd_util_set_rotating_property(Orientation_change.wm, TRUE);
         hd_transition_fade_and_rotate(
-                TRUE, Orientation_change.direction == GOTO_PORTRAIT,
-                G_CALLBACK(hd_transition_rotating_fsm), NULL);
+                        TRUE, Orientation_change.direction == GOTO_PORTRAIT,
+                        G_CALLBACK(hd_transition_rotating_fsm), NULL);
         break;
       case FADE_OUT:
         /*
@@ -1440,6 +1414,16 @@ hd_transition_rotating_fsm(void)
          * to states which don't support the orientation we're
          * going to.
          */
+        /* remove our flag to bodge layout - because we'll rotate properly
+         * soon anyway */
+        Orientation_change.wm->flags &= ~MBWindowManagerFlagLayoutRotated;
+        /* Don't show our screenshot background any more */
+        tidy_cached_group_changed(CLUTTER_ACTOR(hd_render_manager_get()));
+        tidy_cached_group_set_render_cache(
+                                  CLUTTER_ACTOR(hd_render_manager_get()), 0);
+        tidy_cached_group_set_downsampling_factor(
+                                  CLUTTER_ACTOR(hd_render_manager_get()), 0);
+
         state = Orientation_change.goto_state;
         change_state = Orientation_change.new_direction == GOTO_PORTRAIT
           ?  STATE_IS_PORTRAIT(state) || STATE_IS_PORTRAIT_CAPABLE(state)
@@ -1450,17 +1434,6 @@ hd_transition_rotating_fsm(void)
             hd_render_manager_set_state(state);
           }
 
-        /* Now go through our list of waiting effects and complete them */
-        while (Orientation_change.effects_waiting)
-          {
-            hd_transition_completed(0,
-                (HDEffectData*)Orientation_change.effects_waiting->data);
-            Orientation_change.effects_waiting =
-              Orientation_change.effects_waiting->next;
-          }
-        g_list_free(Orientation_change.effects_waiting);
-        Orientation_change.effects_waiting = 0;
-
         if (Orientation_change.direction == Orientation_change.new_direction)
           {
             /*
@@ -1476,7 +1449,9 @@ hd_transition_rotating_fsm(void)
             /* Don't allow anything inside the render manager to tell clutter
              * to redraw. actor_hide should have done this, but it doesn't.
              * We now need to totally blank the screen before the rotation,
-             * so we explicitly call clutter to redraw *right now*. */
+             * so we explicitly call clutter to redraw *right now*. Note that
+             * we don't do this on the stage, because it might conflict with
+             * hd_dbus_system_bus_signal_handler */
             clutter_actor_set_allow_redraw(
                 CLUTTER_ACTOR(hd_render_manager_get()), FALSE);
             clutter_actor_hide(CLUTTER_ACTOR(hd_render_manager_get()));
@@ -1526,6 +1501,14 @@ hd_transition_rotating_fsm(void)
           }
         else /* WAIT_FOR_DAMAGES || FADE_OUT error path */
           { /* Fade back in */
+            /* We must update the layout again so the window sizes
+             * return to normal relative to the screen. flags is probably
+             * already correct. But just for safety. */
+            hd_util_set_screen_size_properties(Orientation_change.wm,
+                        Orientation_change.direction == GOTO_PORTRAIT ? 480 : 800,
+                        Orientation_change.direction == GOTO_PORTRAIT ? 800 : 480);
+            Orientation_change.wm->flags &= ~MBWindowManagerFlagLayoutRotated;
+            mb_wm_layout_update(Orientation_change.wm->layout);
             /* Undo the redraw stopping that happened in FADE_OUT */
             Orientation_change.phase = FADE_IN;
             clutter_actor_set_allow_redraw(
@@ -1610,6 +1593,14 @@ hd_transition_is_rotating (void)
   return Orientation_change.phase != IDLE;
 }
 
+/* Are we about to change to portrait mode? */
+gboolean
+hd_transition_is_rotating_to_portrait (void)
+{
+  return Orientation_change.phase != IDLE
+    && Orientation_change.direction == GOTO_PORTRAIT;
+}
+
 /* Returns whether we are in a state where we should ignore any
  * damage requests. This also checks and possibly prolongs how long
  * we stay in the WAITING state, so we can be sure that all windows
